function adjVal = bound_DC(block_dcfree, adjVal, Tmin, Tmax)
% adjVal_new = bound_DC(block_dcfree, adjVal, Tmin, Tmax)
% 
% This function bounds DC of a given DC-free block.
% The return value is the new adjusted value of this block.
% 
% Default values: Tmin = 0, Tmax = 255.
%
% Copyright (C) 2010 Junaid Jameel Ahmad, Shujun Li (www.hooklee.com)

if ~exist('Tmax', 'var')
    Tmax = 255;
end
if ~exist('Tmin', 'var')
    Tmin = 0;
end
if Tmin>Tmax
    temp = Tmin;
    Tmin = Tmax;
    Tmax = temp;
end
if ~exist('adjVal', 'var')
    adjVal = 0;
end

block_dcfree = block_dcfree(:);

range_min = Tmin-min(block_dcfree);
range_max = Tmax-max(block_dcfree);

if (range_max<range_min) % This should never happen. Issue an alert for debugging.
    beep;
    fprintf('An error occurred: the valid range of DC = [%g, %g]!\n', range_min, range_max);
elseif (adjVal < range_min)
    adjVal = range_min;
elseif (adjVal > range_max)
    adjVal = range_max;
end
