function [img2, range] = bound_DC_image(img, N, Tmin, Tmax)
% img = bound_DC_image(img, N, Tmin, Tmax)
% 
% Bound the DCs of all blocks in the whole image.
% It is actually the process of bounding the DC of the first reference
% block according to the minimal DC range of all the blocks.
% After the first reference block's DC is bounded, the average intensity of
% the whole image is also adjusted accordingly.
%
% Copyright (C) 2010 Shujun Li @ www.hooklee.com

img2 = [];
range = [-Inf Inf];

if nargin<1
    disp('At least one input argument is needed!');
    return;
end

if ~exist('Tmax', 'var')
    Tmax = 255;
end
if ~exist('Tmin', 'var')
    Tmin = 0;
end
if Tmin>Tmax
    temp = Tmin;
    Tmin = Tmax;
    Tmax = temp;
end
if ~exist('N', 'var')
    N = 8;
end

[Y,X] = size(img);
for y=1:N:Y
    for x=1:N:X
        block = img(y:y+N-1,x:x+N-1);
        range = [max(range(1), Tmin-min(block(:))) min(range(2), Tmax-max(block(:)))];
%         if range(1)>range(2)
%             beep;
%         end
    end
end

% Adjust the first reference block and the whole image.
img2 = img + mean(range);
