function img = im_scale_OR_shift(img, Tmin, Tmax)
% img = im_scale_OR_shift(img, Tmin, Tmax)
% 
% Scale or shift an input image to be in the valid range [Tmin, Tmax] if it 
% is out of range.
% If the histogram width is larger than Tmax-Tmin, scale the input image; 
% otherwise shift it towards the valid range until no pixel value is out of
% the range.
%
% Copyright (C) 2010 Shujun Li @ www.hooklee.com

if ~exist('Tmax', 'var')
    Tmax = 255;
end
if ~exist('Tmin', 'var')
    Tmin = 0;
end
if Tmin>Tmax
    temp = Tmin;
    Tmin = Tmax;
    Tmax = temp;
end

img_min = min(img(:));
img_max = max(img(:));
Twidth = Tmax - Tmin;
img_width = img_max-img_min;
if img_width>Twidth
    img = Twidth*(img-img_min)/img_width+Tmin; % Scale.
elseif img_max>Tmax
    img = img + (Tmax-img_max);
elseif img_min<Tmin
    img = img + (Tmin-img_min);
% else % if Tmin<=img_min<=img_max<=Tmax (valid range)
end
