function [img_plain, keystream] = imdecipher(img_cipher, N, Tmax, Tmin, seed, gentype)
% [img_plain, keystream] = imdecipher(img_cipher, N, Tmax, Tmin, seed, gentype)
% 
% This function simulates DC decryption of an input cipher-image with
% MATLAB's rand function.
%
% Input arguments:
% img_cipher: the input image
% N: the size of block DCT (default: 8)
% Tmax, Tmin: the maximal and minimal pixel values (default: 255, 0)
% seed: the seed of the random source (simulating the key of a stream
%       cipher) (default: 5489)
% gentype: type of random stream generator (default: 'mt19937ar')
%
% Shujun Li @ www.hooklee.com 2010

img_plain = [];
keystream = [];

if nargin<1
    disp('At least one input argument is needed!');
    return;
end

if ~exist('gentype','var')
    gentype = 'mt19937ar';
end
if ~exist('seed','var')
    seed = 5489;
end
if ~exist('Tmax', 'var')
    Tmax = 255;
end
if ~exist('Tmin', 'var')
    Tmin = 0;
end
if Tmin>Tmax
    temp = Tmin;
    Tmin = Tmax;
    Tmax = temp;
end
Tmod_1 = Tmax - Tmin;
Tmod = Tmod_1 + 1;
if ~exist('N', 'var')
    N = 8;
end

img_plain = img_cipher - Tmin;

stream = RandStream(gentype, 'Seed', seed);
keystream = zeros(size(img_cipher,1)/N,size(img_cipher,2)/N);
for by=1:N:size(img_cipher,1)
    for bx=1:N:size(img_cipher,2)
        block = img_plain(by:by+N-1,bx:bx+N-1);
        keystream(ceil(by/N),ceil(bx/N)) = randi(stream,[0 Tmod_1]);
        block_mean = mean2(block);
        block_mean_new = mod(block_mean-keystream(ceil(by/N),ceil(bx/N)), Tmod);
        img_plain(by:by+N-1,bx:bx+N-1) = block - block_mean + block_mean_new;
    end
end

img_plain = img_plain + Tmin;
