function [image_list,dir_name,image_number] = imdir(dir_name)
% [image_list,dir_name,image_number] = imdir(dir_name)
% An enhanced edition of dir function for image processing purpose.
% It returns a list of image files supported by MATLAB.
%
% Copyright (C) 2010 Shujun Li @ www.hooklee.com

if (~exist('dir_name','var') || isempty(dir_name)) % dir_name=''
    dir_name = pwd;
end

image_list = dir(dir_name);
if ~isempty(strfind(dir_name, '*'))
    dir_name = fileparts(dir_name); % dir_name is not the folder name!
end
image_list([image_list.isdir]==1) = []; % Remove subfolders.
image_number = numel(image_list);
if image_number<1
    return;
else
    % Remove files which are not images.
    image_invalid_flags = false(1,image_number);
    for i=1:image_number
        try
            imfinfo(fullfile(dir_name, image_list(i).name));
        catch me
            me.message; % Dummy operation to remove warnings.
            image_invalid_flags(i) = true;
        end
    end
    image_list(image_invalid_flags) = [];
    image_number = numel(image_list);
end
