function y = impad(x, N, mode)
% y = impad(x, N, mode)
% 
% This function uses Wavelet Toolbox's function wextend to extend the input
% image x so that there is no incomplete blocks of size N.
% It works only for gray-scale images.
% If the input image x is a color image, it will convert it into a
% gray-scale image via rgb2gray function.
% The default block size is 8 and the default extension method is 'sp0'.
%
% Copyright (C) 2010 Shujun Li @ www.hooklee.com

if nargin<1
    disp('At least one input argument is needed!');
    return;
end

if ~exist('mode','var')
    mode = 'sp0';
end
if ~exist('N','var')
    N = 8;
end

if size(x,3)==3
    y = rgb2gray(x);
else
    y = x(:,:,1);
end
y = double(y);

[Y,X] = size(y);
if mod(Y,N)>0
    n = N - mod(Y,N);
    n_2 = ceil(n/2);
    y = wextend(2, mode, y, [n_2 0]);
    y = y(1:ceil(Y/N)*N,:);
    [Y,X] = size(y);
end
if mod(X,N)>0
    n = N - mod(X,N);
    n_2 = ceil(n/2);
    y = wextend(2, mode, y, [0 n_2]);
    y = y(:,1:ceil(X/N)*N);
end
