function plot_images(varargin)
% plot_images(img, img1, ...)
% plot_images(img, title, img1, title1, ...)
%
% Show one or more images and their histograms.
% The first image is considered as the original image, and its histogram is
% shown in red dahsed line.
% All other histograms are shown in solid lines of different colors.
% PSNR and SSIM are calculated based on double, but histograms are shown by
% treaing all images as uint8 images to make the comparison easier.
%
% Copyright (C) 2010 Shujun Li (www.hooklee.com), Junaid Jameel Ahmad

if nargin<1
    disp('At least one input argument is needed!');
    return;
end

arg_index = 1;
img_index = 1;
imgs = cell(1,nargin);
names = cell(1,nargin);
while arg_index<=nargin
    if isnumeric(varargin{arg_index})
        imgs{img_index} = varargin{arg_index};
        arg_index = arg_index + 1;
        if (arg_index<=nargin && ischar(varargin{arg_index}))
            names{img_index} = varargin{arg_index};
            arg_index = arg_index + 1;
        elseif img_index==1
            names{img_index} = 'Original Image';
        else
            names{img_index} = ['Recovered Image #' num2str(img_index-1)];
        end
        img_index = img_index + 1;
    elseif ischar(varargin{arg_index})
        names{img_index} = varargin{arg_index};
        arg_index = arg_index + 1;
        if arg_index<=nargin
            if isnumeric(varargin{arg_index})
                imgs{img_index} = varargin{arg_index};
                arg_index = arg_index + 1;
                img_index = img_index + 1;
            end
        end
    end
end
imgs(img_index:end) = [];
names(img_index:end) = [];
img_number = img_index - 1;

if img_number<1
    disp('At least one input argument should be an image!');
    return;
end

% Draw the first (original) image.
subplot(2,img_number,1);
imshow(imgs{1},[0 255]);
title(names{1});
% Draw the other image_number-1 images.
for i=2:img_number
    subplot(2,img_number,i);
    imshow(imgs{i},[0 255]);
    title(sprintf('%s: PSNR=%g,\nSSIM=%g, MS-SSIM=%g', ...
        names{i}, PSNR(imgs{i}, imgs{1}), ssim(imgs{i}, imgs{1}), ssim_ms(imgs{i}, imgs{1})));
end
% Draw the histograms.
subplot(2,img_number,img_number+1:2*img_number);
img_hists = zeros(256,img_number);
for i=1:img_number
    img_hists(:,i) = imhist(uint8(imgs{i}));
end
plot(0:255, img_hists(:,1), 'r--');
if img_number>1
    hold on;
    plot(0:255, img_hists(:,2:end));
end
axis tight;
xlabel('Pixel value');
if nargin>1
    ylabel('Histograms');
else
    ylabel('Histogram');
end
legend(names);

if exist('maximize.m','file')==2
    maximize;
end
